@@ -189,4 +189,11 @@ let init_flags () =
     Hashtbl.add int_flags "DELAY_SEMANTIC_EQUIV_INSTANTIATION" 0; (*** by default, no delay, so don't bother evaluating in a model ***)
     Hashtbl.add int_flags "DELAY_TRUE_IN_MODELS" 0; (*** delay processing a proposition based on how often it is true in a model, the idea being that those that are false are more likely to contribute to a refutation ***)
     Hashtbl.add int_flags "DELAY_FALSE_IN_MODELS" 0; (*** delay processing a proposition based on how often it is false in a model ***)
-    Hashtbl.add int_flags "DELAY_UNINFORMATIVE_IN_MODELS" 0 (*** delay those that are true in all models or false in all models ***)
+    Hashtbl.add int_flags "DELAY_UNINFORMATIVE_IN_MODELS" 0; (*** delay those that are true in all models or false in all models ***)
+
+    (*** Dec 2016 ***)
+    Hashtbl.add bool_flags "ONTOLOGY_DEFS_TRANSLUCENT" false;
+    Hashtbl.add int_flags "TRANSLUCENT_EXPAND_DELAY" 5;
+
+    (*** Jan 2017 ***)
+    Hashtbl.add bool_flags "NONFORALL_PATTERN_CLAUSES_USABLE" false
