@@ -3,12 +3,12 @@
 (* Created: September 2010 *)
 
 exception NotFlag of string
-exception NotBoolFlag of string
-exception NotIntFlag of string
 
+val use_learning : bool ref
 val get_bool_flag : string -> bool
 val set_bool_flag : string -> bool -> unit
 val get_int_flag : string -> int
 val set_int_flag : string -> int -> unit
+val set_flag : string -> string -> unit
 val print_flags : unit -> unit
 val init_flags : unit -> unit
