@@ -17,6 +17,13 @@ type metatrm =
    | MLam of stp * metatrm
    | MAp of metatrm * metatrm
 
+let mneg m = MAp(MAp(MGround(Imp),m),MGround(False))
+
+let mneg_p m =
+  match m with
+  | MAp(MAp(MGround(Imp),_),MGround(False)) -> true
+  | _ -> false
+
 type ctx = stp list
 type dpair = ctx * metatrm * trm * stp
 type evar = int * metatrm option ref
@@ -472,7 +479,11 @@ and simpl_dpairs_2 dpairs xl frpairs ffpairs gam m1 m2 b =
 	  | (h2,h2tp,args2) ->
 	      simpl_dpairs dpairs xl ((gam,(m1,x1,sigma1),(m2,h2,h2tp,args2),b)::frpairs) ffpairs
 	end
-    |	_ ->
+    | (_,MAp(MAp(MGround(Imp),MVar(x2,sigma2)),MGround(False))) when not (mneg_p m1) && get_bool_flag "HOUNIF1NEGFLEX" ->
+	simpl_dpairs dpairs xl ((gam,(MVar(x2,sigma2),x2,sigma2),(mneg m1,Imp,Ar(Prop,Ar(Prop,Prop)),[m1;MGround(False)]),b)::frpairs) ffpairs
+    | (MAp(MAp(MGround(Imp),MVar(x1,sigma1)),MGround(False)),_) when not (mneg_p m2) && get_bool_flag "HOUNIF1NEGFLEX" ->
+	simpl_dpairs dpairs xl ((gam,(MVar(x1,sigma1),x1,sigma1),(mneg m2,Imp,Ar(Prop,Ar(Prop,Prop)),[m2;MGround(False)]),b)::frpairs) ffpairs
+    | _ ->
 	begin
 	  match (meta_head_spine m1,meta_head_spine m2) with
 	  | ((h1,h1tp,args1),(h2,h2tp,args2)) ->
@@ -863,22 +874,32 @@ let rec hounif1a m xl xsl gam b p f =
   if (!verbosity > 5) then begin Printf.printf "hounif1 %s gam = %s m = %s\n" (if p then "+" else "-") (match gam with [] -> "[]" | _ -> "...") (metatrm_str m); flush stdout end;
   begin
     match m with
-    | MAp(MAp(MGround(Imp),m1),MGround(False)) -> hounif1a m1 xl xsl gam b (not p) f
-    | MAp(MAp(MGround(Imp),m1),m2) -> hounif1a m1 xl xsl gam b (not p) f; hounif1a m2 xl xsl gam b p f
+    | MAp(MAp(MGround(Imp),m1),MGround(False)) ->
+	if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
+	hounif1a m1 xl xsl gam b (not p) f
+    | MAp(MAp(MGround(Imp),m1),m2) ->
+	if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
+	hounif1a m1 xl xsl gam b (not p) f; hounif1a m2 xl xsl gam b p f
     | MAp(MGround(Forall(a)),(MLam(_,m1))) when p && (match gam with [] -> true | _ -> false) ->
       begin
+	if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
         let (x,xsub) = new_evar_w_default gam a (fun x -> x) in
         hounif1a (metanorm (metasubst m1 0 xsub)) (x::xl) (xsub::xsl) gam b p f
       end
-    | MAp(MGround(Forall(a)),(MLam(_,m1))) -> hounif1a m1 xl xsl (a::gam) b p f
+    | MAp(MGround(Forall(a)),(MLam(_,m1))) ->
+	if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
+	hounif1a m1 xl xsl (a::gam) b p f
     | MAp(MAp(MGround(Eq(Ar(a1,a2))),m1),m2) when p && (match gam with [] -> true | _ -> false) -> (** rigid if negative or gam is nonempty **)
 	begin
+	  if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
           let (x,xsub) = new_evar_w_default gam a1 (fun x -> x) in
 	  hounif1a (metanorm (MAp(MAp(MGround(Eq(a2)),MAp(m1,xsub)),MAp(m2,xsub)))) (x::xl) (xsub::xsl) gam b p f
 	end
     | MAp(MAp(MGround(Eq(Ar(a1,a2))),m1),m2) -> (** rigid if negative or gam is nonempty **)
+	if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
 	hounif1a (metanorm (MAp(MAp(MGround(Eq(a2)),MAp(metashift m1 0 1,MGround(DB(0,a1)))),MAp(metashift m2 0 1,MGround(DB(0,a1)))))) xl xsl (a1::gam) b p f
     | MAp(MAp(MGround(Eq(Prop)),m1),m2) when get_bool_flag "HOUNIF1MATEBELOWEQUIV" ->
+	if get_bool_flag "HOUNIF1MATENONLITS" && gam = [] then mateatom xl xsl m p f;
 	hounif1a m1 xl xsl gam b false f;
 	hounif1a m2 xl xsl gam b false f;
 	hounif1a m1 xl xsl gam b true f;
