@@ -97,17 +97,31 @@ and annotations2_defn_p l =
   | (x::r) -> annotations_defn_p x || annotations2_defn_p r
   | [] -> false
 
+let rec annotation_strings l =
+  match l with
+  | Appl(Symbol "hashroot",[Symbol h]) ->
+      if String.length h = 66 then
+	[("hashroot",String.sub h 1 64)]
+      else
+	raise (Failure ("malformed hashroot " ^ h ^ "; should be '<hash>' where <hash> is 64 characters from [0-9a-f]"))
+  | Appl(Symbol x,r) -> annotation_strings_l r
+  | _ -> []
+and annotation_strings_l l =
+  match l with
+  | l::r -> annotation_strings l @ annotation_strings_l r
+  | [] -> []
+
 let process_thf f =
   match f with
     Appl(Symbol "$$thf",[Symbol name;Symbol role;f;annotations]) ->
       if ((role = "definition") ||
           ((role = "axiom") && (annotations_defn_p annotations)))
       then
-	declare_definition name role (formula_to_pretrm f)
+	declare_definition name role (formula_to_pretrm f) (annotation_strings annotations)
       else if (role = "type") then
-	declare_typed_constant name role (formula_to_pretrm f)
+	declare_typed_constant name role (formula_to_pretrm f) (annotation_strings annotations)
       else
-	declare_thf_logic_formula name role (formula_to_pretrm f)
+	declare_thf_logic_formula name role (formula_to_pretrm f) (annotation_strings annotations)
   | Appl(Symbol "$$include",((Symbol file)::_)) -> (*** ignoring select ***)
       let file2 = String.sub file 1 (String.length file - 2) in (*** Remove the quotes ***)
       let dir2 = Filename.dirname file2 in
