@@ -448,7 +448,7 @@ let slave_cmd s m arg =
 let handle_slave_return pstatus =
   if (!verbosity > 4) then
     Printf.printf "slave returned with status %d\n" pstatus;
-  if (pstatus >= 10) then exit pstatus else if (pstatus = 6) then raise IncompleteSatisfiable else ()
+  if (pstatus >= 10) && (pstatus <= 26) then exit pstatus else if (pstatus = 6) then raise IncompleteSatisfiable else ()
 
 let run_slave final s m =
   let cmd = slave_cmd s m (if final then [] else ["-slave"]) in
