@@ -68,6 +68,8 @@ let get_literal_and_make_pattern_clause m =
 	l
   | _ -> get_literal m
 
+let nonforall_pattern_clauses_usable : bool ref = ref false
+
 (*** Nov 2015: In addition to making the pattern clause when the minisat literal is created, apply it to the pattern clauses that already exist. ***)
 let get_literal_and_make_and_apply_pattern_clause m =
   match m with
@@ -79,7 +81,13 @@ let get_literal_and_make_and_apply_pattern_clause m =
 	new_pattern_clauses l m;
 	apply_pattern_clauses l m;
 	l
-  | _ -> get_literal m
+  | _ ->
+      if !nonforall_pattern_clauses_usable then
+	let l = get_literal m in
+	apply_pattern_clauses l m;
+	l
+      else
+	get_literal m
 
 let sym_eq_clauses_1 mlit msymlit a m1 m2 =
   let symab = Lam(a,forall a (imp (eq a (DB(1,a)) (DB(0,a))) (eq a (DB(0,a)) (DB(1,a))))) in
@@ -1205,26 +1213,43 @@ let process_unprocessed_prop mlit m =
       end
 	
 	
-let process_search_option p op =
+let rec process_search_option p op =
   match op with
   | ProcessProp1(m) ->
       begin
 	if (not (Hashtbl.mem processed m)) then
 	  begin
-	    let mlit = !get_literal_fn2 m in
-	    if ((!not_in_prop_model_delay_p) && (!minisatsearchcounter = 0) && ((minisat_modelValue mlit) > 0)) then (*** Chad: Nov 2011 - delay if not true in the current prop approx; Chad: Dec 2012 - added condition that minisatsearchcounter = 0 so that modelValue is only called if the last interaction with minisat actually called search. Otherwise we may get segmentation faults. The effect is that if MINISAT_SEARCH_PERIOD > 1 and NOT_IN_PROP_MODEL_DELAY > 0, then we only check if the prop is in the model when we happen to have called minisat to search. ***)
-	      begin
-		if (!verbosity > 8) then Printf.printf "using prop model to delay working on %d %s\n" mlit (trm_str m);
-		insertWithPriority (!not_in_prop_model_delay) op
-	      end
-	    else
-	      begin
+	    if !translucent_defns then
+	      let m2 = norm name_def_all m in (*** in case some abbrevs occurring in m are translucent ***)
+	      if m = m2 then
+		process_search_option p (ProcessProp2(m)) (*** none were translucent, the usual case ***)
+	      else (*** some were translucent ***)
+		let mlit = !get_literal_fn2 m in
 		Hashtbl.add processed m p;
-		if (!verbosity > 8) then Printf.printf "working on %d %s\n" mlit (trm_str m);
-		process_unprocessed_prop mlit m
-	      end
+		let m2lit = !get_literal_fn2 m2 in
+		new_search_clause [mlit;-m2lit] (if (mkprooftermp()) then (Some(DeltaRule)) else None);
+		new_search_clause [-mlit;m2lit] (if (mkprooftermp()) then (Some(DeltaRule)) else None);
+		insertWithPriority (get_int_flag "TRANSLUCENT_EXPAND_DELAY") (ProcessProp2(m2))
+	    else
+	      process_search_option p (ProcessProp2(m)) (*** there are no translucent defns at all, so don't even check ***)
 	  end
       end
+  | ProcessProp2(m) -> (*** in this case, assume all translucent defns have been expanded ***)
+      if (not (Hashtbl.mem processed m)) then
+	begin
+	  let mlit = !get_literal_fn2 m in
+	  if ((!not_in_prop_model_delay_p) && (!minisatsearchcounter = 0) && ((minisat_modelValue mlit) > 0)) then (*** Chad: Nov 2011 - delay if not true in the current prop approx; Chad: Dec 2012 - added condition that minisatsearchcounter = 0 so that modelValue is only called if the last interaction with minisat actually called search. Otherwise we may get segmentation faults. The effect is that if MINISAT_SEARCH_PERIOD > 1 and NOT_IN_PROP_MODEL_DELAY > 0, then we only check if the prop is in the model when we happen to have called minisat to search. ***)
+	    begin
+	      if (!verbosity > 8) then Printf.printf "using prop model to delay working on %d %s\n" mlit (trm_str m);
+	      insertWithPriority (!not_in_prop_model_delay) op
+	    end
+	  else
+	    begin
+	      Hashtbl.add processed m p;
+	      if (!verbosity > 8) then Printf.printf "working on %d %s\n" mlit (trm_str m);
+	      process_unprocessed_prop mlit m
+	    end
+	end
   | Mating(plit,nlit,pargs,nargs) ->
       if (!verbosity > 9) then Printf.printf "Mating %d %d\n" plit nlit;
       ignore (mate plit nlit pargs nargs)
@@ -1474,12 +1499,13 @@ let process_and_insert insertfun m =
       let mn = preprocess1 m in
       match (choiceop_axiom mn) with
       | Some(x,a) ->
-	  declare_choiceop x a mn
+	  declare_choiceop x a (mn,m)
       | None ->
 	  begin
 	    insertfun mn;
 	    let mlit = !get_literal_fn1 mn in
-	    new_assumption_clause [mlit];
+	    Hashtbl.add assumption_lit mlit (mn,m);
+	    new_assumption_lit mlit;
 	    if (get_bool_flag "INITIAL_SUBTERMS_AS_INSTANTIATIONS") then
 	      begin
 		if (!verbosity > 4) then Printf.printf "---- Initial Subterms as Instantiations BEGIN ----\n";
@@ -1506,7 +1532,8 @@ let insert_filter_command d = insertWithPriority d (Filter d)
 
 let put_ax_on_queue ax =
   let axlit = !get_literal_fn1 ax in
-  new_assumption_clause [axlit];
+  Hashtbl.add assumption_lit axlit (ax,ax);
+  new_assumption_lit axlit;
   insertWithPriority (get_int_flag "AXIOM_DELAY") (ProcessProp1 ax);
   if (!verbosity > 1) then Printf.printf "Putting axiom on queue %s\n" (trm_str ax)
 
@@ -1568,7 +1595,7 @@ let rec split_global_disjunctions b1 b2 b bs sgn cnj =
 	match m with
 	| Ap(Ap(Imp,Ap(Ap(Eq(_),m1),m2)),False) when (m1 = m2) ->
 	    if (mkprooftermp ()) then
-	      raise (Unsatisfiable (Some NegReflR))
+	      raise (Unsatisfiable (Some (NegReflR(m))))
 	    else
 	      raise (Unsatisfiable None)
 	| Ap(Ap(Imp,Ap(Forall(a),m1)),False) ->
@@ -1690,6 +1717,8 @@ let setup_pattern_clauses () =
   if (get_bool_flag "ENABLE_PATTERN_CLAUSES") then
     begin
       enable_pattern_clauses := true;
+      if (get_bool_flag "NONFORALL_PATTERN_CLAUSES_USABLE") then
+	nonforall_pattern_clauses_usable := true;
       if (get_bool_flag "DYNAMIC_PATTERN_CLAUSES") then
 	dynamic_pattern_clauses := true;
       if (get_bool_flag "PATTERN_CLAUSES_EARLY") then
